@@ -24,6 +24,7 @@ import re
 import sys
 import optparse
 import pkg_resources
+import io
 
 try:
     from collections import OrderedDict
@@ -90,7 +91,7 @@ _html_template = six.u("""<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitio
 <style type="text/css">\n%(style)s\n</style>
 </head>
 <body class="body_foreground body_background" style="font-size: %(font_size)s;" >
-<pre id="content">
+<pre class="ansi2html-content">
 %(content)s
 </pre>
 </body>
@@ -234,7 +235,8 @@ class Ansi2HTMLConverter(object):
         self.ansi_codes_prog = re.compile('\033\\[' '([\\d;]*)' '([a-zA-z])')
 
     def apply_regex(self, ansi):
-        parts = self._apply_regex(ansi)
+        styles_used = set()
+        parts = self._apply_regex(ansi, styles_used)
         parts = self._collapse_cursor(parts)
         parts = list(parts)
 
@@ -249,9 +251,9 @@ class Ansi2HTMLConverter(object):
                 for i, line in enumerate(combined.split('\n'))
             ])
 
-        return combined
+        return combined, styles_used
 
-    def _apply_regex(self, ansi):
+    def _apply_regex(self, ansi, styles_used):
         if self.escaped:
             if self.latex: # Known Perl function which does this: https://tex.stackexchange.com/questions/34580/escape-character-in-latex/119383#119383
                 specials = OrderedDict([
@@ -339,6 +341,7 @@ class Ansi2HTMLConverter(object):
             css_classes = state.to_css_classes()
             if not css_classes:
                 continue
+            styles_used.update(css_classes)
 
             if self.inline:
                 if self.latex:
@@ -376,8 +379,10 @@ class Ansi2HTMLConverter(object):
 
             # Go back, deleting every token in the last 'line'
             if part == CursorMoveUp:
-                final_parts.pop()
-                while '\n' not in final_parts[-1]:
+                if final_parts:
+                    final_parts.pop()
+
+                while final_parts and '\n' not in final_parts[-1]:
                     final_parts.pop()
 
                 continue
@@ -390,7 +395,7 @@ class Ansi2HTMLConverter(object):
     def prepare(self, ansi='', ensure_trailing_newline=False):
         """ Load the contents of 'ansi' into this object """
 
-        body = self.apply_regex(ansi)
+        body, styles = self.apply_regex(ansi)
 
         if ensure_trailing_newline and _needs_extra_newline(body):
             body += '\n'
@@ -399,6 +404,7 @@ class Ansi2HTMLConverter(object):
             'dark_bg': self.dark_bg,
             'font_size': self.font_size,
             'body': body,
+            'styles': styles,
         }
 
         return self._attrs
@@ -418,8 +424,11 @@ class Ansi2HTMLConverter(object):
                 _template = _latex_template
             else:
                 _template = _html_template
+            all_styles = get_styles(self.dark_bg, self.scheme)
+            used_styles = filter(lambda e: e.klass.lstrip(".") in attrs["styles"], all_styles)
+
             return _template % {
-                'style' : "\n".join(map(str, get_styles(self.dark_bg, self.scheme))),
+                'style' : "\n".join(map(str, used_styles)),
                 'title' : self.title,
                 'font_size' : self.font_size,
                 'content' :  attrs["body"],
@@ -513,10 +522,15 @@ def main():
         title=opts.output_title,
     )
 
+    if six.PY3:
+        try:
+            sys.stdin = io.TextIOWrapper(sys.stdin.detach(), opts.input_encoding, "replace")
+        except io.UnsupportedOperation:
+            # This only fails in the test suite...
+            pass
+
     def _read(input_bytes):
         if six.PY3:
-            # This is actually already unicode.  How to we explicitly decode in
-            # python3?  I don't know the answer yet.
             return input_bytes
         else:
             return input_bytes.decode(opts.input_encoding)
